#
# Sketchup Window Settings
#
# -----------------------------------------------------------------------------
#
# Description:
#   SketchUp Window Settings is an interface that allows user to set SketchUp
#   fullscreen, set/remove menubar, show/hide toolbar containers, statusbar,
#   scenesbar, and more. It is a useful tool, especially for those with small
#   monitors.
#
#   Window Settings uses set of Microsoft Windows API functions to interact
#   and retrieve data. The info retrievable functions return window's handle,
#   position, size, etc... The interactive functions modify window's style,
#   position, placement, and visibility state.
#
# -----------------------------------------------------------------------------
#
# Access:
#   (Menu) window -> settings -> show dialog
#
# -----------------------------------------------------------------------------
#
# Usage:
#   Test through menu options and the dialog, you'll figure it out.
#   You may also assign shortcuts, such as F11 to set fullscreen.
#
# -----------------------------------------------------------------------------
#
# Requirements:
#   * Microsoft Windows XP, Vista, 7, 8
#   * Google/Trimble SketchUp 7 or 8 (other versions untested)
#
# -----------------------------------------------------------------------------
#
# Version: 2.0.2
# Release Date: May 9, 2013
#
# -----------------------------------------------------------------------------
#
# Copyright © 2011-2013, Anton Synytsia. ( anton.synytsia@gmail.com )
# License : GNU -- http://opensource.org/licenses/GPL-3.0
#
# You are welcome to use any code snippets for your own plugins!
#
# -----------------------------------------------------------------------------
#
# Thanks to
#   * Daniel J. Berger and Park Heesob for original Win32::API, version 1.4.8.
#   * Dan Rathbun, ThomThom, and Jim for helping recompile Win32::API under my
#     own name-space and for scripting rules.
#   * Aerilius for JS, CSS, and HTML examples from TextureResizer.
#   * Aleksey Synytsia (Dad) for helping with MSFT Windows API.
#   * Gaieus, Rich O Brien, CadFather, feodos, and Burkhard for ideas,
#     suggestions, and for reporting bugs.
#
# -----------------------------------------------------------------------------
#
#   All suggestions and optimizations are welcome!
#   Share 'em here: http://sketchucation.com/forums/viewtopic.php?f=323&t=42926
#
# -----------------------------------------------------------------------------


# Load the normal support files.
require 'sketchup.rb'
require 'extensions.rb'

 # show_ruby_panel

# Check operating system, and make decisions prior to loading.
load_me = true
win = ( Object::RUBY_PLATFORM =~ /mswin/i )
if (win == nil)
  msg = "Hello,\n"
  msg << "It seems your operating system is not MSFT Windows.\n"
  msg << "Sketchup Window Settings plugin requires Windows API to do its task. "
  msg << "Should the plugin be loaded, anyways?"

  input = UI.messagebox(msg, MB_YESNO)
  load_me = false if (input == 7) # NO
end

module AMS  #  <---<<  Anton Synytsia's proprietary toplevel namespace.

  module Sketchup

    # Create the extension.
    @@plugin = SketchupExtension.new 'SU Window Settings', 'ams_WindowSettings/settings dialog.rb'

    desc = ""
    desc << "SketchUp Window Settings is simply an interface that allows user to set SketchUp fullscreen, "
    desc << "show/hide menubar, toolbar container(s), statusbar, scenesbar, and more. "
    desc << "Its a useful tool, especially for those with small monitors."

    # Attach some nice info.
    @@plugin.creator     = 'Anton (anton.synytsia@gmail.com)'
    @@plugin.version     = '2.0.2'
    @@plugin.copyright   = '2011-2013, Anton Synytsia'
    @@plugin.description = desc

    # Register and load the extension on startup.
    Object::Sketchup.register_extension @@plugin, true

  end # module Sketchup

end if load_me
