# -------------------------------------------------------------------------------- #
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# -------------------------------------------------------------------------------- #

Sketchup.require 'sketchup.rb'

dir = File.dirname(__FILE__) # this plugin's folder
Sketchup.require File.join(dir, 'lib/win32/ams_api.so')

module AMS  #  <---<<  Anton Synytsia's proprietary toplevel namespace.

  api = AMS::Win32::API

  orig_verbose = $VERBOSE
  $VERBOSE = nil # Mute warnings caused by constant redefining

  # Platform detection
  OSX = ( Object::RUBY_PLATFORM =~ /darwin/i )
  WIN = ( Object::RUBY_PLATFORM =~ /mswin/i )

  ### ------------------------------------------------------------------------------------------ ###
  ### Microsoft Windows API Reference ###
  ### http://msdn.microsoft.com/en-us/library/aa383749(VS.85).aspx
  ### ------------------------------------------------------------------------------------------ ###

  ### Device Context ### http://msdn.microsoft.com/en-us/library/dd183554(v=vs.85)
  DeleteDC      = api.new('DeleteDC', 'I', 'I', 'Gdi32')
  GetDeviceCaps = api.new('GetDeviceCaps', 'II', 'I', 'Gdi32')
  GetDC         = api.new('GetDC', 'I', 'I', 'User32')
  ReleaseDC     = api.new('ReleaseDC', 'II', 'I', 'User32')

  ### Keyboard Input ### http://msdn.microsoft.com/en-us/library/windows/desktop/ms645530(v=vs.85).aspx
  GetActiveWindow = api.new('GetActiveWindow', 'V', 'I', 'User32')
  GetFocus        = api.new('GetFocus', 'V', 'I', 'User32')
  SetActiveWindow = api.new('SetActiveWindow', 'I', 'I', 'User32')
  SetFocus        = api.new('SetFocus', 'I', 'I', 'User32')

  ### Menu ### http://msdn.microsoft.com/en-us/library/ms646977(v=vs.85).aspx
  GetMenu = api.new('GetMenu', 'I', 'I', 'User32')
  SetMenu = api.new('SetMenu', 'II', 'I', 'User32')

  ### Message and Message Queue ### http://msdn.microsoft.com/en-us/library/windows/desktop/ms632590(v=vs.85).aspx
  PostMessage = api.new('PostMessage', 'ILIP', 'I', 'User32')
  SendMessage = api.new('SendMessage', 'ILIP', 'I', 'User32')

  ### Painting and Drawing ### http://msdn.microsoft.com/en-us/library/dd162760(v=vs.85).aspx
  GetWindowDC      = api.new('GetWindowDC', 'I', 'I', 'User32')
  LockWindowUpdate = api.new('LockWindowUpdate', 'I', 'I', 'User32')
  UpdateWindow     = api.new('UpdateWindow', 'I', 'I', 'User32')

  ### Windows ### http://msdn.microsoft.com/en-us/library/windows/desktop/ms632595(v=vs.85).aspx
  AdjustWindowRectEx       = api.new('AdjustWindowRectEx', 'PLIL', 'I', 'User32')
  AnimateWindow            = api.new('AnimateWindow', 'III', 'I', 'User32')
  BeginDeferWindowPos      = api.new('BeginDeferWindowPos', 'I', 'I', 'User32')
  BringWindowToTop         = api.new('BringWindowToTop', 'I', 'I', 'User32')
  DeferWindowPos           = api.new('DeferWindowPos', 'IIIIIIIL', 'I', 'User32')
  EndDeferWindowPos        = api.new('EndDeferWindowPos', 'I', 'I', 'User32')
  EndTask                  = api.new('EndTask', 'III', 'I', 'User32')
  EnumChildWindows         = api.new('EnumChildWindows', 'IKP', 'I', 'User32')
  EnumThreadWindows        = api.new('EnumThreadWindows', 'IKP', 'I', 'User32')
  EnumWindows              = api.new('EnumWindows', 'KP', 'I', 'User32')
  FindWindow               = api.new('FindWindow', 'PP', 'I', 'User32')
  FindWindowEx             = api.new('FindWindowEx', 'IIPP', 'I', 'User32')
  GetForegroundWindow      = api.new('GetForegroundWindow', 'V', 'I', 'User32')
  GetTopWindow             = api.new('GetTopWindow', 'I', 'I', 'User32')
  GetWindow                = api.new('GetWindow', 'I', 'L', 'User32')
  GetWindowPlacement       = api.new('GetWindowPlacement', 'IP', 'I', 'User32')
  GetWindowRect            = api.new('GetWindowRect', 'IP', 'I', 'User32')
  GetWindowText            = api.new('GetWindowText', 'IPI', 'I', 'User32')
  GetWindowTextLength      = api.new('GetWindowTextLength', 'I', 'I', 'User32')
  GetWindowThreadProcessId = api.new('GetWindowThreadProcessId', 'IP', 'I', 'User32')
  IsIconic                 = api.new('IsIconic', 'I', 'I', 'User32')
  IsZoomed                 = api.new('IsZoomed', 'I', 'I', 'User32')
  IsWindow                 = api.new('IsWindow', 'I', 'I', 'User32')
  IsWindowVisible          = api.new('IsWindowVisible', 'I', 'I', 'User32')
  MoveWindow               = api.new('MoveWindow', 'IIIIII', 'I', 'User32')
  SetForegroundWindow      = api.new('SetForegroundWindow', 'I', 'I', 'User32')
  SetWindowPlacement       = api.new('SetWindowPlacement', 'IP', 'I', 'User32')
  SetWindowPos             = api.new('SetWindowPos', 'IIIIIIL', 'I', 'User32')
  SetWindowText            = api.new('SetWindowText', 'IS', 'I', 'User32')
  ShowOwnedPopups          = api.new('ShowOwnedPopups', 'II', 'I', 'User32')
  ShowWindow               = api.new('ShowWindow', 'II', 'I', 'User32')

  ### Window Class ### http://msdn.microsoft.com/en-us/library/windows/desktop/ms632596(v=vs.85).aspx
  GetClassName  = api.new('GetClassName', 'IPI', 'I', 'User32')
  GetWindowLong = api.new('GetWindowLong', 'II', 'I', 'User32')
  SetWindowLong = api.new('SetWindowLong', 'IIL', 'I', 'User32')

  ### Window Procedures ### http://msdn.microsoft.com/en-us/library/ms632593(v=vs.85).aspx
  DefWindowProc = api.new('DefWindowProc', 'ILIP', 'I', 'User32')

  # Extended Window Style: http://msdn.microsoft.com/en-us/library/windows/desktop/ff700543(v=vs.85).aspx
  # Window Style         : http://msdn.microsoft.com/en-us/library/windows/desktop/ms632600(v=vs.85).aspx

  $VERBOSE = orig_verbose

  class << self  #  <--<< THIS module's anonymous singleton proxy class.
  end # proxy class

end # module AMS
