# -------------------------------------------------------------------------------- #
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# -------------------------------------------------------------------------------- #

require 'sketchup.rb'

dir = File.dirname(__FILE__) # this plugin's folder
require File.join(dir, 'sketchup window.rb')

module AMS::Sketchup::SettingsDialog

  # MODULE VARIABLES
  @@dir = File.dirname(__FILE__)
  @@sdlg = nil
  @@hwnd = nil
  @@first_time ||= true
  @@scenes_obs = nil
  @@std_settings = {
    :set_full_screen  => false,
    :maintain_menubar => true,
    :maintain_viewCE  => true,
    :show_tbc_all     => true,
    :show_tbc_top     => true,
    :show_tbc_bottom  => true,
    :show_tbc_left    => true,
    :show_tbc_right   => true,
    :show_statusbar   => true,
    :show_scenesbar   => true,
    :retain_on_start  => false,
    :set_mb_on_close  => true
  }
  @@saved_settings = @@std_settings

  class << self  #  <--<< THIS module's anonymous singleton proxy class.

    def read_defaults
      return false unless @@first_time
      defaults = eval(Sketchup.read_default("Plugins_ams", "WindowSettings", "{}"))
      @@saved_settings = @@saved_settings.merge(defaults)
      @@first_time = false
      true
    end


    def save_defaults(options)
      @@saved_settings = @@saved_settings.merge(options)
      Sketchup.write_default("Plugins_ams", "WindowSettings", @@saved_settings.inspect.gsub(/"/,"'"))
    end



    def std_settings
      @@std_settings
    end


    def saved_settings
      @@saved_settings.dup
    end


    def get_cur_settings
      amsu = AMS::Sketchup
      cur_settings = {
        :set_full_screen  => amsu::MainWindow.isFullScreen?,
        :maintain_menubar => amsu::MenuBar.there?,
        :maintain_viewCE  => amsu::Viewport.clientEdgeThere?,
        :show_tbc_all     => amsu::ToolbarContainers.visible?(5),
        :show_tbc_top     => amsu::ToolbarContainers.visible?(1),
        :show_tbc_bottom  => amsu::ToolbarContainers.visible?(2),
        :show_tbc_left    => amsu::ToolbarContainers.visible?(3),
        :show_tbc_right   => amsu::ToolbarContainers.visible?(4),
        :show_statusbar   => amsu::StatusBar.visible?,
        :show_scenesbar   => amsu::ScenesBar.visible?,
        :retain_on_start  => @@saved_settings[:retain_on_start],
        :set_mb_on_close  => @@saved_settings[:set_mb_on_close]
      }
      cur_settings
    end


    def set_settings(hash)
      amsu = AMS::Sketchup
      amsu::Viewport.maintainClientEdge(hash[:maintain_viewCE])
      #amsu::ToolbarContainers.show(hash[:show_tbc_all],5)
      amsu::ToolbarContainers.show(hash[:show_tbc_top],1, false)
      amsu::ToolbarContainers.show(hash[:show_tbc_bottom],2, false)
      amsu::ToolbarContainers.show(hash[:show_tbc_left],3, false)
      amsu::ToolbarContainers.show(hash[:show_tbc_right],4, false)
      amsu::StatusBar.show(hash[:show_statusbar], false)
      amsu::ScenesBar.show(hash[:show_scenesbar], false)
      amsu::MainWindow.refresh
      amsu::MenuBar.maintain(hash[:maintain_menubar])
      amsu::MainWindow.setFullScreen(hash[:set_full_screen])
      run if amsu::MenuBar.removed?
      true
    end


    def run
      read_defaults
      settings_dialog
    end


    def settings_dialog
      if (@@sdlg != nil)
        AMS::ShowWindow.call(@@hwnd, 5)
        return false
      end
      amsu = AMS::Sketchup
      dwidth = 320
      dheight = 580
      title = "SU Window Settings"
      @@sdlg = UI::WebDialog.new(title, false, false, dwidth, dheight, 200, 200, true)
      @@sdlg.set_size(dwidth, dheight)
      @@sdlg.set_position(200, 200)
      html_path = File.join(@@dir, 'index.htm')
      @@sdlg.set_file(html_path)
      # Callbacks
      # init: pass the default options to the form
      @@sdlg.add_action_callback("init"){ |dlg, param|
        # Update checkbox inputs with current settings
        get_cur_settings.each{ |k,v|
          k = k.to_s if k.is_a?(Symbol)
          v = v.to_s if v.is_a?(Symbol)
          js_command = "document.getElementById('#{k}').checked = #{v}"
          @@sdlg.execute_script(js_command)
        }
        # Update title text value
        dlg.execute_script("document.getElementById('title_text').value = '#{amsu::MainWindow.caption}'")
        amsu::SettingsObserver.add_observer(self)
        @@scenes_obs = AddSceneTabsObs.new
        Sketchup.active_model.pages.add_observer(@@scenes_obs)
      }
      @@sdlg.add_action_callback("set_caption"){ |dlg, params| amsu::MainWindow.caption = params }
      # Save Button
      @@sdlg.add_action_callback("receiveOptions"){ |dlg, params|
        options = eval(params)
        @@saved_settings = @@saved_settings.merge(options)
      }
      # Retain Button
      @@sdlg.add_action_callback("retain"){set_settings(@@saved_settings)}
      # Reset Button
      @@sdlg.add_action_callback("reset"){set_settings(@@std_settings)}
      # Checkbox clicks
      @@sdlg.add_action_callback("set_full_screen"){amsu::MainWindow.setFullScreen(!amsu::MainWindow.isFullScreen?)}
      @@sdlg.add_action_callback("maintain_menubar"){amsu::MenuBar.maintain(!amsu::MenuBar.there?)}
      @@sdlg.add_action_callback("maintain_viewCE"){amsu::Viewport.maintainClientEdge(!amsu::Viewport.clientEdgeThere?)}
      @@sdlg.add_action_callback("show_tbc_all"){amsu::ToolbarContainers.show(!amsu::ToolbarContainers.visible?(5),5)}
      @@sdlg.add_action_callback("show_tbc_top"){amsu::ToolbarContainers.show(!amsu::ToolbarContainers.visible?(1),1)}
      @@sdlg.add_action_callback("show_tbc_bottom"){amsu::ToolbarContainers.show(!amsu::ToolbarContainers.visible?(2),2)}
      @@sdlg.add_action_callback("show_tbc_left"){amsu::ToolbarContainers.show(!amsu::ToolbarContainers.visible?(3),3)}
      @@sdlg.add_action_callback("show_tbc_right"){amsu::ToolbarContainers.show(!amsu::ToolbarContainers.visible?(4),4)}
      @@sdlg.add_action_callback("show_statusbar"){amsu::StatusBar.show(!amsu::StatusBar.visible?)}
      @@sdlg.add_action_callback("show_scenesbar"){amsu::ScenesBar.show(!amsu::ScenesBar.visible?)}
      @@sdlg.add_action_callback("retain_on_start_receiver"){ |dlg, params|
        @@saved_settings[:retain_on_start] = (eval(params) == true)
      }
      @@sdlg.add_action_callback("set_mb_on_close_receiver"){ |dlg, params|
        @@saved_settings[:set_mb_on_close] = (eval(params) == true)
      }
      @@sdlg.add_action_callback("show_dialogs"){amsu.showDialogs}
      @@sdlg.add_action_callback("hide_dialogs"){amsu.showDialogs(false)}
      @@sdlg.add_action_callback("close_dialogs"){amsu.closeDialogs()}
      @@sdlg.add_action_callback("show_ftoolbars"){amsu.showFloatingToolbars}
      @@sdlg.add_action_callback("hide_ftoolbars"){amsu.showFloatingToolbars(false)}
      @@sdlg.add_action_callback("close_ftoolbars"){amsu.closeFloatingToolbars()}
      # Set on close
      @@sdlg.set_on_close(){
        amsu::SettingsObserver.remove_observer(self)
        Sketchup.active_model.pages.remove_observer(@@scenes_obs)
        @@scenes_obs = nil
        amsu::MenuBar.maintain if @@saved_settings[:set_mb_on_close]
        amsu.includeDialog(@@hwnd)
        @@sdlg = nil
        @@hwnd = nil
      }
      # Show dialog
      @@sdlg.show
      @@hwnd = amsu.getWindowByCaption(title)
      amsu.ignoreDialog(@@hwnd)
      true
    end


    ### AMS Sketchup Observers ###

    def onCaptionChanged(text)
      js_command = "document.getElementById('title_text').value = '#{text}'"
      @@sdlg.execute_script(js_command)
    end


    def onSetFullScreen(status)
      js_command = "document.getElementById('set_full_screen').checked = #{status}"
      @@sdlg.execute_script(js_command)
    end


    def onSetMenuBar(status)
      # Show settings dialog if the menubar is removed; a way back.
      run if (status == false)
      js_command = "document.getElementById('maintain_menubar').checked = #{status}"
      @@sdlg.execute_script(js_command)
    end


    def onSetViewCE(status)
      js_command = "document.getElementById('maintain_viewCE').checked = #{status}"
      @@sdlg.execute_script(js_command)
    end


    def onShowToolbarContainers(status, bar)
      tbc = AMS::Sketchup::ToolbarContainers
      js_command = "document.getElementById('show_tbc_top').checked = #{tbc.visible?(1)};
      document.getElementById('show_tbc_bottom').checked = #{tbc.visible?(2)};
      document.getElementById('show_tbc_left').checked = #{tbc.visible?(3)};
      document.getElementById('show_tbc_right').checked = #{tbc.visible?(4)};
      document.getElementById('show_tbc_all').checked = #{tbc.visible?(5)};"
      @@sdlg.execute_script(js_command)
    end


    def onShowStatusBar(status)
      js_command = "document.getElementById('show_statusbar').checked = #{status}"
      @@sdlg.execute_script(js_command)
    end


    def onShowScenesBar(status)
      js_command = "document.getElementById('show_scenesbar').checked = #{status}"
      @@sdlg.execute_script(js_command)
    end

  end # proxy class


  # ScenesBar Extended Observer
  # Include this observer to keep track of scene tabs.
  class AddSceneTabsObs < Object::Sketchup::PagesObserver

    def onElementAdded(cpages, page)
      amsu = Object::AMS::Sketchup
      timer = UI.start_timer(0.1, false){
        UI.stop_timer(timer)
        amsu::SettingsDialog.onShowScenesBar(amsu::ScenesBar.visible?)
      }
    end


    def onElementRemoved(cpages, page)
      amsu = Object::AMS::Sketchup
      timer = UI.start_timer(0.1, false){
        UI.stop_timer(timer)
        amsu::SettingsDialog.onShowScenesBar(amsu::ScenesBar.visible?)
      }
    end

  end # class AddSceneTabsObs


  # Include app observer to save settings on close.
  class MyAppObserver < Sketchup::AppObserver

    def onQuit
      sdlg = Object::AMS::Sketchup::SettingsDialog
      sdlg.save_defaults(sdlg.saved_settings)
    end

  end # class MyAppObserver


  unless file_loaded?(__FILE__)

    amsu = Object::AMS::Sketchup
    timer = UI.start_timer(0.5, false){
      UI.stop_timer(timer)
      self.read_defaults
      self.set_settings(self.saved_settings) if self.saved_settings[:retain_on_start]
      self.run if amsu::MenuBar.removed?
    }

    Sketchup.add_observer(MyAppObserver.new)

    # Add menu items
    submenu = UI.menu("Window").add_submenu("Settings")

    item = submenu.add_item("set fullscreen"){
      state = !amsu::MainWindow.isFullScreen?
      amsu::MainWindow.setFullScreen state
    }
    submenu.set_validation_proc(item){ amsu::MainWindow.isFullScreen? ? MF_CHECKED : MF_UNCHECKED }

    item = submenu.add_item("maintain menubar"){
      state = !amsu::MenuBar.there?
      amsu::MenuBar.maintain state
      self.run if amsu::MenuBar.removed?
    }
    submenu.set_validation_proc(item){ amsu::MenuBar.there? ? MF_CHECKED : MF_UNCHECKED }

    item = submenu.add_item("maintain view border"){
      state = !amsu::Viewport.clientEdgeThere?
      amsu::Viewport.maintainClientEdge state
    }
    submenu.set_validation_proc(item){ amsu::Viewport.clientEdgeThere? ? MF_CHECKED : MF_UNCHECKED }

    item = submenu.add_item("show toolbar containers"){
      state = !amsu::ToolbarContainers.visible?
      amsu::ToolbarContainers.show state
    }
    submenu.set_validation_proc(item){ amsu::ToolbarContainers.visible? ? MF_CHECKED : MF_UNCHECKED }

    item = submenu.add_item("show statusbar"){
      state = !amsu::StatusBar.visible?
      amsu::StatusBar.show state
    }
    submenu.set_validation_proc(item){ amsu::StatusBar.visible? ? MF_CHECKED : MF_UNCHECKED }

    item = submenu.add_item("show scenesbar"){
      state = !amsu::ScenesBar.visible?
      amsu::ScenesBar.show state
    }
    submenu.set_validation_proc(item){ amsu::ScenesBar.visible? ? MF_CHECKED : MF_UNCHECKED }

    # Include Show Dialog, Save, Retain, and Reset
    submenu.add_separator
    item = submenu.add_item("show dialog"){ self.run }
    submenu.add_item("save"){ self.save_defaults self.get_cur_settings }
    submenu.add_item("retain"){ self.set_settings self.saved_settings }
    submenu.add_item("reset"){ self.set_settings self.std_settings }

    file_loaded(__FILE__)

  end # unless file_loaded?(__FILE__)

end # module AMS::Sketchup::SettingsDialog
