#Sketchup.require 'ams_WindowSettings/sketchup window.rb'

# Menu bar nested namespace.
# Menu bar is a strip with commands.
# It is usually located at the top of SketchUp window.
module AMS::Sketchup::MenuBar

  class << self  #  <--<< THIS module's anonymous singleton proxy class.

    # Retrieves hWnd to menu bar
    def hwnd
      AMS::Sketchup.menu_bar
    end

    alias handle hwnd


    # Menu bar cannot be shown or hidden; it can only be set or removed.
    # Warning, removing menu bar disables all menu hot key operations!
    # Args::
    #   Bool: true - set, false - remove
    # Returns::
    #   Bool: true - state changed, false - state not changed
    #
    def maintain(state = true)
      state = [true,1].include?(state)
      hMenu = (state ? hwnd : nil)
      return false if (state == there?)
      AMS::SetMenu.call(AMS::Sketchup.main_window, hMenu)
      AMS::Sketchup::SettingsObserver.onSetMenuBar(state)
      true
    end


    # Same as <tt>maintain( true )</tt>
    def set; maintain(true) end


    # Same as <tt>maintain( false )</tt>
    def remove; maintain(false) end


    # Removes menu bar for the specified time in seconds.
    def removeForTime(sec = 60)
      sec = sec.abs
      maintain false
      t = Object::UI.start_timer(sec, false){
        Object::UI.stop_timer t
        maintain true
      }
      nil
    end


    # Retrieves status, whether current menu bar is set or removed.
    def there?; (AMS::GetMenu.call(AMS::Sketchup.main_window) == hwnd) end

    alias set? there?


    # Same as <tt> not there?()</tt>
    def removed?; !there? end

  end # proxy class

end # module AMS::Sketchup::MenuBar
