#Sketchup.require 'ams_WindowSettings/sketchup window.rb'

# SettingsObserver nested namespace.
# SettingsObserver is a module containing numerous callback functions, each reacting to certain event.
# Please use only <tt>add_observer</tt>, and <tt>remove_observer</tt> methods.
# Don't call any other of the observers unless you changed the state manually, via your own function.
module AMS::Sketchup::SettingsObserver

  # MODULE VARIABLES
  @@instances ||= []

  class << self  #  <--<< THIS module's anonymous singleton proxy class.

    # This method is used to add an observer.
    def add_observer(inst)
      return false if @@instances.include?(inst)
      @@instances.push(inst)
      true
    end


    # This method is used to remove an observer.
    def remove_observer(inst)
      (@@instances.delete(inst) != nil) # will return true/false
    end


    # This observer is called when Sketchup parent window's title text was changed.
    # text: the new title string
    def onCaptionChanged(text)
      @@instances.each{ |inst|
        name = "onCaptionChanged"
        inst.method(name).call(text) if inst.methods.include?(name)
      }
    end


    # status: -true- main window was set full screen, -false- main window was reset to original placement
    def onSetFullScreen(status)
      @@instances.each{ |inst|
        name = "onSetFullScreen"
        inst.method(name).call(status) if inst.methods.include?(name)
      }
    end


    # status: -true- menubar was set, -false- menubar was removed
    def onSetMenuBar(status)
      @@instances.each{ |inst|
        name = "onSetMenuBar"
        inst.method(name).call(status) if inst.methods.include?(name)
      }
    end


    # status: -true- bar was set visible, -false- bar was set hidden
    # bar: 1 - top, 2 - bottom, 3 - left, 4 - right, 5 - all
    def onShowToolbarContainers(status, bar)
      @@instances.each{ |inst|
        name = "onShowToolbarContainers"
        inst.method(name).call(status, bar) if inst.methods.include?(name)
      }
    end


    # status: -true- bar was set visible, -false- bar was set hidden
    def onShowStatusBar(status)
      @@instances.each{ |inst|
        name = "onShowStatusBar"
        inst.method(name).call(status) if inst.methods.include?(name)
      }
    end


    # status: -true- bar was set visible, -false- bar was set hidden
    def onShowScenesBar(status)
      @@instances.each{ |inst|
        name = "onShowScenesBar"
        inst.method(name).call(status) if inst.methods.include?(name)
      }
    end


    # status: -true- view border was set, -false- view border was removed
    def onSetViewCE(status)
      @@instances.each{ |inst|
        name = "onSetViewCE"
        inst.method(name).call(status) if inst.methods.include?(name)
      }
    end

  end # proxy class

end # module AMS::Sketchup::SettingsObserver
