#Sketchup.require 'ams_WindowSettings/sketchup window.rb'

# Toolbar containers nested namespace.
# Toolbar containers are placeholders for floating toolbars.
# SketchUp has four toolbar containers, surrounding the rectengular viewport.
# They are also called control bars.
module AMS::Sketchup::ToolbarContainers

  class << self  #  <--<< THIS module's anonymous singleton proxy class.

    # Retrieves an array of hWnds to all four toolbar containers
    def hwnds
      AMS::Sketchup.control_bars.dup
    end

    alias handles hwnds


    # Retrives hWnd to the top toolbar container
    def top_bar; hwnds[0] end


    # Retrives hWnd to the bottom toolbar container
    def bottom_bar; hwnds[1] end


    # Retrives hWnd to the left toolbar container
    def left_bar; hwnds[2] end


    # Retrives hWnd to the right toolbar container
    def right_bar; hwnds[3] end


    # This function is used to show/hide the window.
    # Args::
    #   bar: 1 - top, 2 - bottom, 3 - left, 4 - right, 5 - all
    #   state: true - show, false - hide
    # Returns::
    #   Bool: true - state changed, false - state not changed
    #
    def show(state = true, bar = 5, refresh = true)
      state = [true,1].include?(state)
      refresh = [true,1].include?(refresh)
      nCmdShow = (state ? 5 : 0)
      if bar.is_a?(Object::String)
        bar = ['top', 'bottom', 'left', 'right', 'all'].index(bar)
        bar = (bar == nil) ? 5 : bar+1
      else
        bar = 5 unless [1,2,3,4,5].include?(bar)
      end
      wind = case bar
      when 1 : hwnds[0]
      when 2 : hwnds[1]
      when 3 : hwnds[2]
      when 4 : hwnds[3]
      when 5 :
        count = 0
        hwnds.each{ |hwnd|
          next if (state == (AMS::IsWindowVisible.call(hwnd) != 0))
          AMS::ShowWindow.call(hwnd, nCmdShow)
          count += 1
        }
        # Return true if at least ones visibility state changed; otherwise, false.
        return false if (count == 0)
        AMS::Sketchup::MainWindow.refresh if refresh
        AMS::Sketchup::SettingsObserver.onShowToolbarContainers(state, bar)
        return true
      end # case bar
      return false if (wind == nil) or (state == (AMS::IsWindowVisible.call(wind) != 0))
      AMS::ShowWindow.call(wind, nCmdShow)
      AMS::Sketchup::MainWindow.refresh if refresh
      AMS::Sketchup::SettingsObserver.onShowToolbarContainers(state, bar)
      true
    end


    # This function used to determine whether the window is visible.
    # Args::
    #   bar: 1 - top, 2 - bottom, 3 - left, 4 - right, 5 - all
    # Returns::
    #   Bool: true - visible, false - some or all bars are not visible
    #
    def visible?(bar = 5)
      hwnd = case bar
      when 1, 'top'   : hwnds[0]
      when 2, 'bottom': hwnds[1]
      when 3, 'left'  : hwnds[2]
      when 4, 'right' : hwnds[3]
      when 5, 'all'
        count = 0
        hwnds.each{ |hwnd| count += 1 if (AMS::IsWindowVisible.call(hwnd) != 0) }
        # Return true if all four are visible; otherwise, false.
        return (count == 4)
      end # case bar
      return false unless hwnd
      (AMS::IsWindowVisible.call(hwnd) != 0)
    end

  end # proxy class

end # module AMS::Sketchup::ToolbarContainers
