#Sketchup.require 'ams_WindowSettings/sketchup window.rb'

# Viewport nested namespace.
# Viewport is SketchUp view window, where all drawings take place.
module AMS::Sketchup::Viewport

  class << self  #  <--<< THIS module's anonymous singleton proxy class.

    # Retrieves hWnd to view window
    def hwnd
      AMS::Sketchup.viewport
    end

    alias handle hwnd


    # This function is used to get the dimensions of the bounding rectangle of the window.
    # The dimensions are given in screen coordinates that are relative to the upper-left corner of the screen.
    # It retrieves the screen coordinates of the upper-left and lower-right corners of the window.
    # Returns an array: [x1,y1, x2,y2]
    def rect
      lpRect = 0.chr*16
      AMS::GetWindowRect.call(hwnd, lpRect)
      lpRect.unpack('l*')
    end


    # Retrieves the screen coordinates of the upper-left corner of the window
    # Returns an array: [x,y]
    def origin; rect[0..1] end


    # Retrieves window's size in pixels
    # Returns an array: [width,height]
    def size
      r = rect
      w = r[2] - r[0]
      h = r[3] - r[1]
      [w, h]
    end


    # Retrieves window's width
    def width; size[0] end


    # Retrieves window's height
    def height; size[1] end


    # This function is used to get the center position of the window in screen coordinates.
    # Returns an array: [cx,cy]
    def center
      p = rect
      cx = (p[0] + p[2]) / 2
      cy = (p[1] + p[3]) / 2
      [cx, cy]
    end


    # This function is used to get the position of all four corners relative to screen coordinates.
    # It retrieves the screen coordinates of the upper-left, upper-right, lower-left, and lower-right corners of the window.
    # Returns an array: [[x1,y1], [x2,y2], [x3,y3], [x4,y4]]
    def corners
      p = rect
      [[p[0],p[1]], [p[2],p[1]], [p[0],p[3]], [p[2],p[3]]]
    end


    # This function is used to set/remove view client edge, a thin line border.
    def maintainClientEdge(state = true)
      state = [true,1].include?(state)
      exstyle = state ? 0x200 : 0
      return false if (state == clientEdgeThere?)
      AMS::LockWindowUpdate.call hwnd
      AMS::SetWindowLong.call(hwnd, -20, exstyle)
      AMS::LockWindowUpdate.call nil
      AMS::SetWindowPos.call(hwnd, 0, 0, 0, 0, 0, 0x0277)
      AMS::Sketchup::SettingsObserver.onSetViewCE(state)
      true
    end


    # This function is used to determine whether the view window has a thin line border.
    def clientEdgeThere?
      exstyle = AMS::GetWindowLong.call(hwnd, -20)
      ((exstyle | 0x00000200) == exstyle)
    end


    # This function is used to repaint the window.
    def repaint
      AMS::PostMessage.call(hwnd, 0x000F, 0, 0)
      hdc = AMS::GetWindowDC.call(hwnd)
      AMS::DefWindowProc.call(hwnd, 0x0014, hdc, 0)
      AMS::ReleaseDC.call(hwnd, hdc)
    end

  end # proxy class

end # module AMS::Sketchup::Viewport
